@@ -16,7 +16,6 @@ use crate::{
     bytecode::{encode_str, write_to_vec},
     collections::key_map::{Handle, KeyMap},
     compiled_program::{CaoCompiledProgram, Label},
-    instruction::instruction_span,
     prelude::TraceEntry,
     Instruction, NodeId, VariableId,
 };
@@ -151,49 +150,50 @@ impl<'a> Compiler<'a> {
         self.current_card = -1;
         for (i, n) in compilation_unit.iter().enumerate() {
             self.current_lane = n.name.clone();
-
-            let indexkey = Handle::from_i64(i as i64);
-            assert!(!self.jump_table.contains(indexkey));
             num_cards += n.cards.len();
 
-            let nodekey = Handle::from_u32(
+            let nodekey = Handle::from_u64(
                 NodeId {
-                    // we know that i fits in 16 bits from the check above
-                    lane: i as u16,
+                    lane: i
+                        .try_into()
+                        .map_err(|_| self.error(CompilationErrorPayload::TooManyLanes))?,
                     pos: 0,
                 }
                 .into(),
             );
-            // allow referencing lanes using both name and index
-            let metadata = LaneMeta {
-                hash_key: nodekey,
-                arity: n.arguments.len() as u32,
-            };
-            self.jump_table.insert(indexkey, metadata).unwrap();
-            let namekey = Handle::from_str(n.name.as_str()).expect("Failed to hash lane name");
-            if self.jump_table.contains(namekey) {
-                return Err(self.error(CompilationErrorPayload::DuplicateName(n.name.clone())));
-            }
-            self.jump_table.insert(namekey, metadata).unwrap();
+            self.add_lane(nodekey, n)?;
         }
 
         self.program.labels.0.reserve(num_cards).expect("reserve");
         Ok(())
     }
 
+    fn add_lane(&mut self, nodekey: Handle, n: &CompiledLane) -> CompilationResult<()> {
+        let metadata = LaneMeta {
+            hash_key: nodekey,
+            arity: n.arguments.len() as u32,
+        };
+        let namekey = Handle::from_str(n.name.as_str()).expect("Failed to hash lane name");
+        if self.jump_table.contains(namekey) {
+            return Err(self.error(CompilationErrorPayload::DuplicateName(n.name.clone())));
+        }
+        self.jump_table.insert(namekey, metadata).unwrap();
+        Ok(())
+    }
+
     /// consume lane cards and build the bytecode
     fn compile_stage_2(&mut self, compilation_unit: CaoIr<'a>) -> CompilationResult<()> {
         let mut lanes = compilation_unit.iter().enumerate();
 
         if let Some((il, main_lane)) = lanes.next() {
-            let len: u16 = match main_lane.cards.len().try_into() {
+            let len: u32 = match main_lane.cards.len().try_into() {
                 Ok(i) => i,
                 Err(_) => return Err(self.error(CompilationErrorPayload::TooManyCards(il))),
             };
             self.scope_begin();
             self.process_lane(il, main_lane, 0)?;
             let nodeid = NodeId {
-                lane: il as u16,
+                lane: il as u32,
                 pos: len,
             };
             self.scope_end();
@@ -203,10 +203,10 @@ impl<'a> Compiler<'a> {
 
         for (il, lane) in lanes {
             let nodeid = NodeId {
-                lane: il as u16,
+                lane: il as u32,
                 pos: 0,
             };
-            let nodeid_hash = Handle::from_u32(nodeid.into());
+            let nodeid_hash = Handle::from_u64(nodeid.into());
             let handle = u32::try_from(self.program.bytecode.len())
                 .expect("bytecode length to fit into 32 bits");
             self.program
@@ -217,7 +217,6 @@ impl<'a> Compiler<'a> {
 
             self.scope_begin();
 
-            // process the lane
             self.process_lane(il, lane, 1)?;
 
             self.scope_end();
@@ -249,15 +248,18 @@ impl<'a> Compiler<'a> {
     }
 
     /// add a local variable
-    fn add_local(&mut self, name: &'a str) -> CompilationResult<()> {
+    ///
+    /// return its index
+    fn add_local(&mut self, name: &'a str) -> CompilationResult<u32> {
         self.validate_var_name(name)?;
+        let result = self.locals.len();
         self.locals
             .try_push(Local {
                 name,
                 depth: self.scope_depth,
             })
             .map_err(|_| self.error(CompilationErrorPayload::TooManyLocals))?;
-        Ok(())
+        Ok(result as u32)
     }
 
     fn process_lane(
@@ -269,7 +271,6 @@ impl<'a> Compiler<'a> {
             cards,
             namespace,
         }: &'a CompiledLane,
-        // cards: Vec<Card>,
         instruction_offset: i32,
     ) -> CompilationResult<()> {
         self.current_lane = name.clone();
@@ -288,18 +289,19 @@ impl<'a> Compiler<'a> {
         for (ic, card) in cards.iter().enumerate() {
             self.current_card = ic as i32;
             let nodeid = NodeId {
-                lane: il as u16,
-                pos: (ic as i32 + instruction_offset) as u16,
+                lane: il as u32,
+                pos: (ic as i32 + instruction_offset) as u32,
             };
             self.process_card(nodeid, card)?;
         }
         Ok(())
     }
 
-    fn conditional_jump(
+    /// encodes the then block (card), and conditionally jumps over it using the `skip_instr`
+    fn encode_if_then(
         &mut self,
         skip_instr: Instruction,
-        lane: &LaneNode,
+        then: impl FnOnce(&mut Self) -> CompilationResult<()>,
     ) -> CompilationResult<()> {
         type Pos = i32;
         assert!(
@@ -310,11 +312,16 @@ impl<'a> Compiler<'a> {
             "invalid skip instruction"
         );
         self.push_instruction(skip_instr);
-        let mut pos = instruction_span(Instruction::CallLane) + self.program.bytecode.len() as Pos;
-        pos += mem::size_of_val(&pos) as Pos;
-        write_to_vec(pos, &mut self.program.bytecode);
-        self.push_instruction(Instruction::CallLane);
-        self.encode_jump(lane)?;
+        let idx = self.program.bytecode.len();
+        write_to_vec(0 as Pos, &mut self.program.bytecode);
+
+        // write the `then` block
+        then(self)?;
+
+        unsafe {
+            let ptr = self.program.bytecode.as_mut_ptr().add(idx) as *mut Pos;
+            std::ptr::write(ptr, self.program.bytecode.len() as Pos);
+        };
         Ok(())
     }
 
@@ -363,7 +370,7 @@ impl<'a> Compiler<'a> {
     fn process_card(&mut self, nodeid: NodeId, card: &'a Card) -> CompilationResult<()> {
         let handle = u32::try_from(self.program.bytecode.len())
             .expect("bytecode length to fit into 32 bits");
-        let nodeid_hash = Handle::from_u32(nodeid.into());
+        let nodeid_hash = Handle::from_u64(nodeid.into());
         self.program
             .labels
             .0
@@ -424,8 +431,7 @@ impl<'a> Compiler<'a> {
                 self.read_var_card(variable)?;
             }
             Card::SetVar(var) => {
-                let index = self.locals.len() as u32;
-                self.add_local(&*var.0)?;
+                let index = self.add_local(&*var.0)?;
                 self.push_instruction(Instruction::SetLocalVar);
                 write_to_vec(index, &mut self.program.bytecode);
             }
@@ -454,41 +460,31 @@ impl<'a> Compiler<'a> {
                 write_to_vec(*id, &mut self.program.bytecode);
             }
             Card::IfElse {
-                then: then_lane,
-                r#else: else_lane,
+                then: then_card,
+                r#else: else_card,
             } => {
-                // if true jump to then (2nd item) else execute 1st item then jump over the 2nd
-                self.push_instruction(Instruction::GotoIfTrue);
-                let pos = instruction_span(Instruction::Goto)
-                    + instruction_span(Instruction::CallLane)
-                    + self.program.bytecode.len() as i32
-                    + 4; // +4 == sizeof pos
-                debug_assert_eq!(std::mem::size_of_val(&pos), 4);
-                write_to_vec(pos, &mut self.program.bytecode);
-                // else
-                self.push_instruction(Instruction::CallLane);
-                self.encode_jump(else_lane)?;
-
-                self.push_instruction(Instruction::Goto);
-                let pos = instruction_span(Instruction::CallLane)
-                    + self.program.bytecode.len() as i32
-                    + 4; // +4 == sizeof pos
-                write_to_vec(pos, &mut self.program.bytecode);
-                // then
-                self.push_instruction(Instruction::CallLane);
-                self.encode_jump(then_lane)?;
+                let mut idx = 0;
+                self.encode_if_then(Instruction::GotoIfFalse, |c| {
+                    c.process_card(nodeid, then_card)?;
+                    // jump over the `else` branch
+                    c.push_instruction(Instruction::Goto);
+                    idx = c.program.bytecode.len();
+                    write_to_vec(0i32, &mut c.program.bytecode);
+                    Ok(())
+                })?;
+                self.process_card(nodeid, else_card)?;
+                unsafe {
+                    let ptr = self.program.bytecode.as_mut_ptr().add(idx) as *mut i32;
+                    std::ptr::write(ptr, self.program.bytecode.len() as i32);
+                }
             }
             Card::IfFalse(jmp) => {
-                // if the value is true we DON'T jump
-                self.conditional_jump(Instruction::GotoIfTrue, jmp)?;
+                self.encode_if_then(Instruction::GotoIfTrue, |c| c.process_card(nodeid, jmp))?
             }
             Card::IfTrue(jmp) => {
-                // if the value is false we DON'T jump
-                self.conditional_jump(Instruction::GotoIfFalse, jmp)?;
-            }
-            Card::Jump(jmp) => {
-                self.encode_jump(jmp)?;
+                self.encode_if_then(Instruction::GotoIfFalse, |c| c.process_card(nodeid, jmp))?
             }
+            Card::Jump(jmp) => self.encode_jump(jmp)?,
             Card::StringLiteral(c) => self.push_str(c.0.as_str()),
             Card::CallNative(c) => {
                 let name = &c.0;
@@ -554,13 +550,16 @@ impl<'a> Compiler<'a> {
             write_to_vec(id, &mut self.program.bytecode);
         } else {
             //local
-            self.push_instruction(Instruction::ReadLocalVar);
-            let index = scope as u32;
-            write_to_vec(index, &mut self.program.bytecode);
+            self.read_local_var(scope as u32);
         }
         Ok(())
     }
 
+    fn read_local_var(&mut self, index: u32) {
+        self.push_instruction(Instruction::ReadLocalVar);
+        write_to_vec(index, &mut self.program.bytecode);
+    }
+
     fn validate_var_name(&self, name: &str) -> CompilationResult<()> {
         if name.is_empty() {
             return Err(self.error(CompilationErrorPayload::EmptyVariable));
